--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **            Project name: TMeas                                       **
-- **            Module: TMeas (root)                                      **
-- **            Target device: CoolRunner II (XC2C256/TQ144) [DIGILENT]   **
-- **            Tool versions: ISE WebPACK 9.2i                           **
-- **            Date: BIAYSTOK, stycze 2009                             **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity TMeas is port ( Clk : in std_logic ;
                       Reset : in std_logic ;
                       InpSignal : in std_logic ;
                       GoKey : in std_logic ;
                       RunStatusLed : out std_logic ;
                       MeasLed : out std_logic ;
                       SignalEcho : out std_logic ;
                       Seg_A : out std_logic ;
                       Seg_B : out std_logic ;
                       Seg_C : out std_logic ;
                       Seg_D : out std_logic ;
                       Seg_E : out std_logic ;
                       Seg_F : out std_logic ;
                       Seg_G : out std_logic ;
                       Seg_K : out std_logic ;
                       DSel0 : out std_logic ;
                       DSel1 : out std_logic ;
                       DSel2 : out std_logic ;
                       DSel3 : out std_logic ) ;
end TMeas ;
--
--             --------+                            +---------------
--                     |                            |
--                     |<========== t =============>|
--                     |                            |
--                     +----------------------------+
--

architecture Behavioral of TMeas is

  component PushButton port ( KClk : in std_logic ;
                              Reset : in std_logic ;
                              KeyIn : in std_logic ;
                              Status : out std_logic ) ;
  end component ;

  component BCD4DigCnt port ( Clk : in std_logic ;
                              Reset : in std_logic ;
                              Enable : in std_logic ;
                              Cnt1 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt10 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt100 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt1000 : out std_logic_vector ( 3 downto 0 ) ) ;
  end component ;

  component Mux4DigDisp port ( DClk : in std_logic ;
                               Reset : in std_logic ;
                               Neg : in std_logic := '0' ;
                               WA : in std_logic := '0' ;
                               Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                               Dot : in std_logic_vector ( 3 downto 0 ) ;
                               Seg_A : out std_logic ;
                               Seg_B : out std_logic ;
                               Seg_C : out std_logic ;
                               Seg_D : out std_logic ;
                               Seg_E : out std_logic ;
                               Seg_F : out std_logic ;
                               Seg_G : out std_logic ;
                               Seg_K : out std_logic ;
                               DSel0 : out std_logic ;
                               DSel1 : out std_logic ;
                               DSel2 : out std_logic ;
                               DSel3 : out std_logic ) ;
  end component ;

  type AutomatStateType is ( ResetState ,
                             IdleState ,
                             StartMeasState ,
                             WaitOnStartSignalHighState ,
                             WaitOnStartMeasState ,
                             WaitOnStopMeasState ,
                             StopMeasState ) ;

  signal AutomatState : AutomatStateType ;
  signal FDiv : std_logic_vector ( 11 downto 0 ) ;
  signal TimeDiv : std_logic_vector ( 9 downto 0 ) ;
  signal F1kHz : std_logic ;
  signal GoKeyStatus : std_logic ;
  signal DClk : std_logic ;
  signal KClk : std_logic ;
  signal Enable : std_logic ;
  signal Clear : std_logic ;
  signal ClrCnt : std_logic ;
  signal Cnt1 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt10 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt100 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt1000 : std_logic_vector ( 3 downto 0 ) ;

begin
  TimeDivInstance : process ( Clk , ClrCnt , Enable )
  begin
    if ClrCnt = '0' then
      TimeDiv <= "0000000000" ;
    else
      if Enable = '1' then
        if Clk'event and Clk = '1' then
          if TimeDiv = "1111100111" then
            TimeDiv <= "0000000000" ;
          else
            TimeDiv <= TimeDiv + 1 ;
          end if ;
        end if ;
      end if ;
    end if ;
  end process ;
  F1kHz <= TimeDiv ( 9 ) ;

  FDivInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      FDiv <= "000000000000" ;
    else
      if Clk'event and Clk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  KClk <= FDiv ( 11 ) ;
  DClk <= FDiv ( 9 ) ;

  SignalEcho <= not InpSignal ;
  MeasLed <= not Enable ;

  AutomatInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      AutomatState <= ResetState ;
     else
      if Clk'event and Clk = '1' then
        case AutomatState is
          when ResetState =>
            Clear <= '0' ;
            Enable <= '0' ;
            RunStatusLed <= '1' ;
            AutomatState <= IdleState ;
          when IdleState =>
            Clear <= '1' ;
            Enable <= '0' ;
            if GoKeyStatus = '0' then
              AutomatState <= IdleState ;
            else
              AutomatState <= StartMeasState ;
            end if ;
          when StartMeasState =>
            Clear <= '0' ;
            Enable <= '0' ;
            RunStatusLed <= '0' ;
            if InpSignal = '0' then
              AutomatState <= WaitOnStartSignalHighState ;
            else
              AutomatState <= WaitOnStartMeasState ;
            end if ;
          when WaitOnStartSignalHighState =>
            Clear <= '1' ;
            Enable <= '0' ;
            if InpSignal = '0' then
              AutomatState <= WaitOnStartSignalHighState ;
            else
              AutomatState <= WaitOnStartMeasState ;
            end if ;
          when WaitOnStartMeasState =>
            Clear <= '1' ;
            Enable <= '0' ;
            if InpSignal = '1' then
              AutomatState <= WaitOnStartMeasState ;
            else
              AutomatState <= WaitOnStopMeasState ;
            end if ;
          when WaitOnStopMeasState =>
            Clear <= '1' ;
            Enable <= '1' ;
            if InpSignal = '0' then
              AutomatState <= WaitOnStopMeasState ;
            else
              AutomatState <= StopMeasState ;
            end if ;
          when StopMeasState =>
            Clear <= '1' ;
            Enable <= '0' ;
            RunStatusLed <= '1' ;
            AutomatState <= IdleState ;
        end case ;
      end if ;
    end if ;
  end process ;

  ClrCnt <= '0' when ( Clear = '0' or Reset = '0' ) else '1' ;

  Key0Instance : PushButton port map ( KClk => KClk ,
                                       Reset => Reset ,
                                       KeyIn => GoKey ,
                                       Status => GoKeyStatus ) ;

  BCDCntInstance : BCD4DigCnt port map ( Clk => F1kHz ,
                                         Reset => ClrCnt ,
                                         Enable => Enable ,
                                         Cnt1 => Cnt1 ,
                                         Cnt10 => Cnt10 ,
                                         Cnt100 => Cnt100 ,
                                         Cnt1000 => Cnt1000 ) ;

  MuxDisplInstance : Mux4DigDisp port map ( DClk => DClk ,
                                            Reset => Reset ,
                                            Tetr0 => Cnt1 ,
                                            Tetr1 => Cnt10 ,
                                            Tetr2 => Cnt100 ,
                                            Tetr3 => Cnt1000 ,
                                            Dot => "0111" ,
                                            Seg_A => Seg_A ,
                                            Seg_B => Seg_B ,
                                            Seg_C => Seg_C ,
                                            Seg_D => Seg_D ,
                                            Seg_E => Seg_E ,
                                            Seg_F => Seg_F ,
                                            Seg_G => Seg_G ,
                                            Seg_K => Seg_K ,
                                            DSel0 => DSel0 ,
                                            DSel1 => DSel1 ,
                                            DSel2 => DSel2 ,
                                            DSel3 => DSel3 ) ;

end Behavioral ;
